"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const src_1 = require("../src");
const Bank_1 = __importDefault(require("../src/structures/Bank"));
const LootTable_1 = __importDefault(require("../src/structures/LootTable"));
const Monster_1 = __importDefault(require("../src/structures/Monster"));
const testUtil_1 = require("./testUtil");
(0, vitest_1.describe)('Monsters', () => {
    const currentMonIDs = new Set();
    const currentAliases = new Set();
    const subSubTable = new LootTable_1.default().add('Coal');
    const quantityTable = new LootTable_1.default().add('Dragon claws');
    const emptyTable = new LootTable_1.default({ limit: 100 }).add('Rune crossbow');
    const subTable = new LootTable_1.default()
        .add('Needle')
        .add('Amethyst')
        .add('Knife')
        .add([['Iron bar'], ['Steel bar']])
        .add(subSubTable);
    (0, vitest_1.beforeAll)(() => __awaiter(void 0, void 0, void 0, function* () {
        for (const monster of src_1.Monsters.values()) {
            if (!monster.aliases.some(alias => alias === monster.name.toLowerCase())) {
                throw `${monster.name} should have its name as an alias.`;
            }
            // Make sure all aliases are lowercase
            for (const alias of monster.aliases) {
                if (alias.toLowerCase() !== alias) {
                    throw `Aliases for ${monster.name} should be lowercased.`;
                }
                if (currentAliases.has(alias))
                    throw `Alias of '${alias}' is duplicated.`;
                currentAliases.add(alias);
            }
            if (currentMonIDs.has(monster.id))
                throw `${monster.name} has the same ID as another monster.`;
            currentMonIDs.add(monster.id);
        }
    }));
    class TestMonsterClass extends Monster_1.default {
        constructor() {
            super(...arguments);
            Object.defineProperty(this, "table", {
                enumerable: true,
                configurable: true,
                writable: true,
                value: new LootTable_1.default()
                    .every('Dragon bones')
                    .every('Bones')
                    .tertiary(100, 'Ranger boots')
                    .tertiary(10, 'Twisted bow')
                    .add('Bandos page 1')
                    .add('Bandos page 2')
                    .add('Bandos page 3')
                    .add('Bandos page 4')
                    .add(subTable)
                    .add(quantityTable, 100)
                    .add(emptyTable)
            });
        }
        kill(quantity = 1) {
            const loot = new Bank_1.default();
            for (let i = 0; i < quantity; i++) {
                const roll = this.table.roll();
                const barDrop = roll.amount('Iron bar');
                const otherBarDrop = roll.amount('Steel bar');
                if (otherBarDrop !== barDrop) {
                    throw new Error('Should drop equal amount');
                }
                const dragonClawsAmount = roll.amount('Dragon claws');
                if (dragonClawsAmount !== 0 && dragonClawsAmount !== 100) {
                    throw new Error('Dragon claws amount must be 0 or 100');
                }
                loot.add(roll);
            }
            return loot;
        }
    }
    (0, vitest_1.test)('Test Monster', () => {
        const TesterMonster = new TestMonsterClass({ id: 1, name: 'Test' });
        const number = 500000;
        const expectedRates = {
            'Dragon bones': 1,
            Bones: 1,
            'Ranger boots': 100,
            'Twisted bow': 10,
            'Bandos page 1': TesterMonster.table.length,
            'Bandos page 2': TesterMonster.table.length,
            'Bandos page 3': TesterMonster.table.length,
            'Bandos page 4': TesterMonster.table.length,
            Knife: TesterMonster.table.length * subTable.length,
            Amethyst: TesterMonster.table.length * subTable.length,
            Needle: TesterMonster.table.length * subTable.length,
            Coal: (TesterMonster.table.length * subTable.length) / subSubTable.length,
            'Iron bar': TesterMonster.table.length * subTable.length,
            'Steel bar': TesterMonster.table.length * subTable.length,
            'Dragon claws': TesterMonster.table.length / 100,
            'Rune crossbow': TesterMonster.table.length * 100
        };
        const loot = TesterMonster.kill(number);
        return (0, testUtil_1.checkThreshold)(expectedRates, loot, number);
    });
    (0, vitest_1.test)('Duplicate IDs', () => {
        const ids = [];
        for (const monster of src_1.Monsters.values()) {
            if (ids.includes(monster.id))
                throw new Error(`${monster.id} is duplicated`);
            ids.push(monster.id);
        }
    });
});
//# sourceMappingURL=Monsters.test.js.map